# Serial port - Octave comunication
# aztk CopyLeft 2010
#
# s = serial(portname,OPTIONS)
# Start a connection Octave - Python through a local socket
#  and Python is connected to the serial port

function info_port = serial(portname,varargin)

pkg load sockets

if !nargin
	usage("serial('COM1','BaudRate',9600,'DataBits',8,'Parity','N','StopBits',1,'timeout','None','xonxoff',0,'rtscts',0)");
endif

% Default values
%portname = portname; % Device name
baudrate = '9600'; % Baud rate such as 9600 or 115200 etc
bytesize = '8'; % Number of data bits. Possible values: 5 6 7 8
parity = 'N'; % Enable parity checking. Possible values: 'N'(None) 'O'(Odd) 'E'(Even)
stopbits = '1'; % Number of stop bits. Possible values: 1 2
timeout = 'None'; % Set a read timeout value, in seconds
xonxoff = '0'; % Enable software flow control
rtscts = '0'; % Enable hardware (RTS/CTS) flow control

% Two arguments
% s = serial('/dev/ttyUSB0',9600)
if nargin == 2
	if isnumeric(varargin{1})
		baudrate = varargin{1};
	else
		error('BaudRate debe ser un entero')
	endif
endif

% Three arguments or more
% s = serial('/dev/ttyUSB0','BaudRate',9600)
% s = serial('/dev/ttyUSB0','timeout',1.5)
% s = serial('/dev/ttyUSB0','StopBits',2,'BAUD',9600,'timeout','None')
if nargin > 2
	for k=1:nargin-2
		%spoption = varargin{k};
		valueop = varargin{k+1};
		
		switch varargin{k}
			case { 'BaudRate', 'BAUD', 'baudrate' }
			if isnumeric(valueop)
				baudrate = num2str(valueop);
			else
				error('BaudRate debe ser un entero')
			endif
			
			case { 'DataBits' }
			if ((valueop < 5) && (valueop > 8))
				error('DataBits { 5 6 7 8 }')
			else
				bytesize = num2str(valueop);
			endif
			
			case { 'Parity' }
			if ((valueop != 'E') && (valueop != 'N') && (valueop != 'O'))
				error('Parity {E N O}')
			else
				parity = valueop;
			endif
			
			case { 'StopBits' }
			if ((valueop != 1) && (valueop != 2))
				error('StopBits {1 2}')
			else
				stopbits = num2str(valueop);
			endif
			
			case { 'timeout' }
			if ((valueop == 'None') || (isnumeric(valueop)))
				timeout = num2str(valueop);
			else
				error('timeout {None | <x>} x in seconds (float allowed)')
			endif
			
			case { 'xonxoff' }
			if valueop
				xonxoff = num2str(1);
			else
				xonxoff = num2str(0);
			endif
			
			case { 'rtscts' }
			if valueop
				rtscts = num2str(1);
			else
				rtscts = num2str(0);
			endif
		endswitch
	endfor
endif

options_sport=[portname,' ',baudrate,' ',bytesize,' ',parity,' ',stopbits,' ',timeout,' ',xonxoff,' ',rtscts];

% To create a octave-socket
% s=socket(AF_INET,SOCK_STREAM,0)
s=socket(2,1,0);

addr_socket=9999; % Initial octave-socket address
n=5; % Total connection attempts

for i=addr_socket:-1:addr_socket-n
  sts=bind(s,i); % Octave-socket as server
  if ~sts
    break
  endif
endfor
if i==addr_socket-n && sts
  error('No socket address available')
%  sts=-1;
%  return
endif
addr_socket=num2str(i);

% Only one client
sts=listen(s,1);
if sts
  error('Error')
%  sts=-2;
%  return
endif

% Launch the program client
sts=system(['START sport_client.py',' ',addr_socket,' ',options_sport]);
if sts
  error(error_code)
%  sts=-3;
%  return
endif
pause(0.1)
% To accept connection from the client
c=accept(s);

sts=char(recv(c,6));
if sts != '%stok%'
  disconnect(c)
  disconnect(s)
  error('Serial port busy or no exists')
%  sts=-4;
%  return
endif

% Return a struct with Device and Server information
info_port = struct('streamAddress',c,'streamServerAddress',s,'socketPort',i,'portName', portname, 'BaudRate', baudrate, 'DataBits', bytesize, 'Parity', parity, 'StopBits', stopbits, 'timeout', timeout, 'xonxoff', xonxoff, 'rtscts', rtscts);
sts=0;
return

endfunction